#ifndef ACCOUNT_H
#define ACCOUNT_H

#include <QObject>
#include <QMetaType>

#include "OviMusicExport.h"
#include "DownloadCommand.h"

namespace Ovi {

class RedeemCreditPinCommand;
class PurchaseSubscriptionCommand;
class GetStreamUrlsCommand;
class StreamUrlReleases;

/*!
 * \class Account
 *
 * Holds the account information.
 */
class OVIMUSICAPISHARED_EXPORT Account : public QObject
{
    Q_OBJECT

    Q_PROPERTY(double creditBalance READ creditBalance NOTIFY creditBalanceChanged)

public:
    ~Account();

    /*!
     * Redeem a credit pin.
     * \param pin Voucher PIN code.
     */
    Q_INVOKABLE virtual Ovi::RedeemCreditPinCommand* redeemCreditPin(const QString &pin) = 0;

    /*!
     * Purchase a subscription.
     * \param subscriptionId Id of the subscription to purchase.
     */
    Q_INVOKABLE virtual Ovi::PurchaseSubscriptionCommand* purchaseSubscription(const QString &subscriptionId) = 0;

    /*!
     * Download release.
     *
     * \param releaseId Release to download
     * \param usage Should be a value from DownloadCommand::UsageType.  This is an int because of an issue in QML. 
     */
    Q_INVOKABLE virtual Ovi::DownloadCommand* download(const QString &releaseId, int usage) = 0;

    /*!
     * Get full-length stream URLs for specific playlists, tracks and collections.
     * \param releases Releases to get full-length stream urls to.
     */
    Q_INVOKABLE virtual Ovi::GetStreamUrlsCommand* getStreamUrls(Ovi::StreamUrlReleases *releases) = 0;

    /*!
     * Get the user's credit balance.  Tries to keep track of the user's balance as calls are made.  To 
     * make sure the balance is accurate, call refreshCreditBalance().
     */
    virtual double creditBalance() const = 0;

    /*!
     * Retrieve the user's credit balance from the music service.  When finished
     * the creditBalanceChanged signal is emitted if the balance has changed.
     */
    Q_INVOKABLE virtual void refreshCreditBalance() = 0;

signals:
    void creditBalanceChanged(double balance);

protected:
    Account(QObject *parent = 0);
};

} /* end Ovi namespace */

Q_DECLARE_METATYPE(Ovi::Account*)

#endif // ACCOUNT_H
