#ifndef IAUTHENTICATION_H
#define IAUTHENTICATION_H

#include <QtCore/QObject>
#include <QMetaType>

#include "OviMusicExport.h"

namespace Ovi
{
    /// Represents all the functions for Authentication
    class IAuthentication
    {
    public:
        /// Possible error states
        enum ErrorStatus
        {
            errorNone = 0,
            errorUnknown,
            errorSignInCancelled,
            errorAccountNotConfigured
        };
        
        enum TokenState
        {
            tokenStateRenewed, ///< Token renewed.
            tokenStateExpired, ///< Token expired.
            tokenStateDeleted ///< Token deleted.
            /*Please add enums only after the last line*/
        };

        /// Allow conversion to a QObject for signals/slots to be connected being as
        /// this objejct doesn't derive from QObject in order to simplify its use in plugins
        virtual operator QObject*() = 0;

        /// Destructor
        virtual ~IAuthentication() { }

    public:
        /// Retrieve the token for this Ovi application.
        /// - Identifies session
        ///
        /// @see tokenResponse for retrieving token as a signal
        ///
        virtual void token() = 0;

        /// Create Signature using Token Key & Token Secret
        /// available with NoA Auth Plugin over the passed
        /// baseMessage.
        ///
        /// @param baseMsg Base message to be used for creating
        /// signature
        ///
        /// @see signatureResponse for retrieving signature as a signal
        ///
        virtual void signature(const QString& baseMsg) = 0;

        /// User name of the signed in account as stored in the SSO backend.
        /// - Usually a GUID, but always unique to a given user
        ///
        /// @see userIdResponse for retrieving the user Id
        ///
        virtual void userId() = 0;

        /// Returns the current SSO Backend timestamp
        /// The api has to be used only after sign in.
        /// Calling this API without signing in will return current time.
        ///
        /// This value shall be in seconds.
        ///
        /// @see time Since oAuthTimeStampResponse signal for retrieving timestamp
        ///
        virtual void oAuthTimeStamp() = 0;

    signals:
        /// Signal emitted when signIn request is completed against
        /// NoA Server. If any error has occured, errorCode is passed
        /// and application should take necessary action.
        ///
        /// @param errorCode Error code if any during initilisation.
        virtual void signInResponse(ErrorStatus status) = 0;

        /// Token for this Ovi application is returned. If any
        /// error is encountered, errorCode is populated.
        ///
        /// This token is valid till the session of application is
        /// active with this Interface layer. NoA Auth Plugin
        /// will keep on refreshing the validity of the token keeping
        /// token value same.
        ///
        /// @param errorCode Error code if any during retrieving token.
        /// @param token Token for this application.
        ///
        virtual void tokenResponse(ErrorStatus status, const QString token) = 0;

        /// Signature created on the baseMessage passed by Ovi Application.
        /// If any error is encountered, errorCode is populated & passed.
        ///
        /// @param errorCode Error code if any during retrieving token.
        /// @param signature Signature created.
        ///
        virtual void signatureResponse(ErrorStatus status, const QString signature) = 0;

        /// User name of the signed in account as stored in the SSO backend.
        ///
        /// @param errorCode Errorcode if any during retrieving data.
        /// @param username user name of the loggedin identifier if exists else returns NULL.
        ///
        virtual void userIdResponse(ErrorStatus status, const QString username) = 0;

        /// Time (in Seconds) SSO server backend timestamp
        ///
        /// @param errorCode Errorcode if any during retrieving data..
        /// @param oAuthTimeInSeconds
        ///
        virtual void oAuthTimeStampResponse(ErrorStatus status, const QString oAuthTimeInSeconds) = 0;

        /// Signal indicating status of Signout request.
        /// On successful signout, UI Application can update the display status to offline.
        ///
        /// @param errorCode Error code if any during signing out .
        ///
        virtual void signOutResponse(ErrorStatus status) = 0;

        /**
        * Notification to the client, about any change in the state of the access token 
        * e.g. if token has expired or been deleted.
        *
        * @param TokenState Indicates current state of token.
        */
        virtual void tokenStateChanged(TokenState state) = 0;

    }; // IAuthentication

} // End Ovi Namespace

Q_DECLARE_METATYPE(Ovi::IAuthentication::ErrorStatus);
Q_DECLARE_METATYPE(Ovi::IAuthentication::TokenState);

#endif // I_AUTHENTICATION_H
