#ifndef ITEM_H
#define ITEM_H

#include <QUrl>
#include <QDate>
#include <QVariant>
#include <QScopedPointer>

#include "OviMusicExport.h"

namespace Ovi {

class ICallBuilderFactory;
class StoreItemList;
class Genre;
class ItemPrivate;
class GetStreamUrlsCommand;
class Account;

/*!
 * \class Item
 *
 * Class representing an abstract music item.
 *
 * Metadata can be accessed through the Item's properties.
 *
 * Recommendations based on the Item can be gotten through
 * recommendedTracks() and recommendedAlbums().
 */
class OVIMUSICAPISHARED_EXPORT Item : public QObject
{
    friend class ItemFactory;

    Q_OBJECT

    Q_PROPERTY(QString type READ type CONSTANT)
    Q_PROPERTY(QString id READ id CONSTANT)
    Q_PROPERTY(QUrl uri READ uri CONSTANT)
    Q_PROPERTY(QString name READ name CONSTANT)
    Q_PROPERTY(QString category READ category CONSTANT)
    Q_PROPERTY(int duration READ duration CONSTANT)
    Q_PROPERTY(QString label READ label CONSTANT)
    Q_PROPERTY(bool parentalAdvisory READ parentalAdvisory CONSTANT)
    Q_PROPERTY(QString copyright READ copyright CONSTANT)
    Q_PROPERTY(QString artistId READ artistId CONSTANT)
    Q_PROPERTY(QString artistName READ artistName CONSTANT)
    Q_PROPERTY(bool variousArtists READ variousArtists CONSTANT)
    Q_PROPERTY(QDate streetReleaseDate READ streetReleaseDate CONSTANT)
    Q_PROPERTY(QString takenFromId READ takenFromId CONSTANT)
    Q_PROPERTY(QString takenFrom READ takenFrom CONSTANT)
    Q_PROPERTY(QUrl albumArt50 READ albumArt50 CONSTANT)
    Q_PROPERTY(QUrl albumArt100 READ albumArt100 CONSTANT)
    Q_PROPERTY(QUrl albumArt200 READ albumArt200 CONSTANT)
    Q_PROPERTY(QUrl storeUri READ storeUri CONSTANT)
    Q_PROPERTY(bool cwm READ cwm CONSTANT)
    Q_PROPERTY(bool alc READ alc CONSTANT)
    Q_PROPERTY(QString priceCurrency READ priceCurrency CONSTANT)
    Q_PROPERTY(QString priceValue READ priceValue CONSTANT)
    Q_PROPERTY(int trackCount READ trackCount CONSTANT)
    Q_PROPERTY(int cwmBitRate READ cwmBitRate CONSTANT)
    Q_PROPERTY(QString cwmFileType READ cwmFileType CONSTANT)
    Q_PROPERTY(int alcBitRate READ alcBitRate CONSTANT)
    Q_PROPERTY(QString alcFileType READ alcFileType CONSTANT)
    Q_PROPERTY(Ovi::Genre* genre READ genre CONSTANT)
    Q_PROPERTY(QUrl wmaClip READ wmaClip CONSTANT)
    Q_PROPERTY(QUrl mp3Clip READ mp3Clip CONSTANT)

public:
    virtual ~Item() = 0;

    QString type() const;
    QString id() const;
    QUrl uri() const;
    QString name() const;
    QString category() const;
    int duration() const;
    QString label() const;
    bool parentalAdvisory() const;
    QString copyright() const;
    virtual QString artistId() const;
    virtual QString artistName() const;
    bool variousArtists() const;
    QString genreId() const;
    QString genreName() const;
    QDate streetReleaseDate() const;
    QString takenFromId() const;
    QString takenFrom() const;
    QUrl takenFromUri() const;
    QUrl albumArt50() const;
    QUrl albumArt100() const;
    QUrl albumArt200() const;
    QUrl wmaClip() const;
    QUrl mp3Clip() const;
    QUrl storeUri() const;
    bool cwm() const;
    bool alc() const;
    QString priceCurrency() const;
    QString priceValue() const;
    int trackCount() const;
    int cwmBitRate() const;
    QString cwmFileType() const;
    int alcBitRate() const;
    QString alcFileType() const;

    Q_INVOKABLE virtual Ovi::StoreItemList* recommendedTracks();
    Q_INVOKABLE virtual Ovi::StoreItemList* recommendedAlbums();
    Q_INVOKABLE virtual Ovi::StoreItemList* tracks();

    Genre* genre() const;

    Q_INVOKABLE virtual GetStreamUrlsCommand* fullLengthStreamUrls() const;

protected:
    Item(const QSharedPointer<ICallBuilderFactory> &callFactory, const Account* account, const QVariantMap& properties);

    QSharedPointer<ICallBuilderFactory> callFactory() const;
    const Account* account() const;
    QVariantMap properties() const;

private:
    QScopedPointer<ItemPrivate> d;
};

} /* end Ovi namespace */

Q_DECLARE_METATYPE(Ovi::Item*)

#endif // ITEM_H
