#ifndef SEARCHREQUEST_H
#define SEARCHREQUEST_H

#include "OviMusicExport.h"

#include <QObject>
#include <QMetaType>
#include <QSharedDataPointer>
#include <QStringList>

namespace Ovi {

class SearchRequestPrivate;

/*!
 * \class SearchRequest
 *
 * Holds the information for a search request to be sent by Store.
 */
class OVIMUSICAPISHARED_EXPORT SearchRequest : public QObject
{
    Q_OBJECT

    Q_ENUMS( OrderBy );
    Q_ENUMS( SortOrder );
    Q_FLAGS( Category Categories );
    Q_ENUMS( SearchMode );
    Q_PROPERTY( QString searchText READ searchText WRITE setSearchText )
    Q_PROPERTY( OrderBy orderBy READ orderBy WRITE setOrderBy )
    Q_PROPERTY( SortOrder sortOrder READ sortOrder WRITE setSortOrder )
    Q_PROPERTY( Categories categories READ categories WRITE setCategories )
    Q_PROPERTY( SearchMode searchMode READ searchMode WRITE setSearchMode )
    Q_PROPERTY( int itemsPerPage READ itemsPerPage WRITE setItemsPerPage )

public:
    /*!
     * \enum OrderBy
     *
     * Criteria to order the search results by.
     */
    enum OrderBy
    {
        Relevancy,
        Name,
        StreetReleaseDate,
        AllTimeDownloads,
        RecentDownloads
    };

    /*!
     * \enum SortOrder
     *
     * Order to sort the search results in.  This may not have any effect depending on the OrderBy setting.
     */
    enum SortOrder
    {
        DefaultOrder,
        AscendingOrder,
        DescendingOrder
    };

    /*!
     * \enum Category
     *
     * Categories to filter search results by.
     */
    enum Category
    {
        UnspecifiedCategory = 0x00,
        Album = 0x01,
        Single = 0x02,
        Track = 0x04,
        AllCategories = Album | Single | Track
    };
    Q_DECLARE_FLAGS( Categories, Category );

    /*!
     * \enum SearchMode
     *
     * Controls the strictness of the query.
     */
    enum SearchMode
    {
        Accurate, /*!< Will not return mis-spelt suggestions. */
        Forgiving /*!< Will return any mis-spelt suggestions. */
    };

    SearchRequest(QObject *parent=0);
    ~SearchRequest();

    Q_INVOKABLE Ovi::SearchRequest *clone(QObject *parent=0) const;

    bool isValid() const;

    void setSearchText(const QString &text);
    QString searchText() const;

    void setOrderBy(OrderBy order);
    OrderBy orderBy() const;

    void setSortOrder(SortOrder order);
    SortOrder sortOrder() const;

    void setCategories(Categories cat);
    Categories categories() const;

    void setSearchMode(SearchMode mode);
    SearchMode searchMode() const;

    void setItemsPerPage(int count);
    int itemsPerPage() const;

    void setIds(const QStringList& ids);
    QStringList ids() const;

    static void setItemsPerPageDefault(int count);
    static int itemsPerPageDefault();
    static int itemsPerPageMin();
    static int itemsPerPageMax();

private:
    SearchRequest(const SearchRequest *other, QObject *parent=0);
    QSharedDataPointer<SearchRequestPrivate> d;
};

} /* end Ovi namespace */

Q_DECLARE_METATYPE(Ovi::SearchRequest*)

#endif //SEARCHREQUEST_H
