#ifndef OVI_STORE_H
#define OVI_STORE_H

#include <QObject>
#include <QSharedPointer>

#include "OviMusicExport.h"
#include "StoreInfo.h"

namespace Ovi {

class StoreItemList;
class SearchRequest;
class GenreList;
class Account;
class ChartsList;
class RequestStatus;
class Playlists;
class Subscriptions;

/*!
 * \class Store
 *
 * \brief Provides access to the catalogue of an Ovi Music Store
 */
class OVIMUSICAPISHARED_EXPORT Store : public QObject
{
    friend class StoreFactory;

    Q_OBJECT

    Q_PROPERTY(QString version READ version)

    Q_PROPERTY(Ovi::Account* account READ account CONSTANT)
    Q_PROPERTY(Ovi::StoreItemList* newAlbumReleases READ newAlbumReleases)
    Q_PROPERTY(Ovi::StoreItemList* newSingleRelease READ newSingleRelease)
    Q_PROPERTY(Ovi::StoreItemList* newTrackRelease READ newTrackRelease)
    Q_PROPERTY(Ovi::StoreItemList* topAlbums READ topAlbums)
    Q_PROPERTY(Ovi::StoreItemList* topTracks READ topTracks)
    Q_PROPERTY(Ovi::RequestStatus * status READ status CONSTANT)

public:
    Store(QObject *parent=0);

    Q_INVOKABLE virtual QString version() const = 0;
    virtual StoreInfo storeInfo() const = 0;

    /*!
     * Get a list of genres available for this store.
     *
     * The returned GenreList is owned by the store.
     *
     * If called before the store is ready, returns 0.
     */
    Q_INVOKABLE virtual Ovi::GenreList *genres() = 0;

    /*!
     * Get a list of charts available for this store.
     *
     * The returned ChartsList is owned by the store.
     *
     * If called before the store is ready, returns 0.
     */
    Q_INVOKABLE virtual Ovi::ChartsList *charts() = 0;

    /*!
     * Perform a search using the specified request.
     *
     * Only the searchText, orderBy and sortOrder properties of the request are used for this search.
     *
     * \return a StoreItemList which receives the search results
     */
    Q_INVOKABLE virtual Ovi::StoreItemList* search(Ovi::SearchRequest *request) = 0;

    /*!
     * Search for artists using the specified request.
     *
     * Only the searchText, orderBy and sortOrder properties of the request are used for this search.
     *
     * \return a StoreItemList which contains Artist items.
     */
    Q_INVOKABLE virtual Ovi::StoreItemList *searchForArtists(Ovi::SearchRequest *request) = 0;
    Q_INVOKABLE virtual Ovi::StoreItemList* searchFiltered(const QString& artist, const QString& album, const QString& track) = 0;
    Q_INVOKABLE virtual Ovi::StoreItemList* searchArtist(const QString& artist) = 0;
    Q_INVOKABLE virtual Ovi::StoreItemList* searchAlbum(const QString& album) = 0;
    Q_INVOKABLE virtual Ovi::StoreItemList* searchTrack(const QString& track) = 0;
    Q_INVOKABLE virtual Ovi::StoreItemList* searchUpcCode(const QString& upcCode) = 0;

    Q_INVOKABLE virtual Ovi::StoreItemList* release(const QString& releaseId) = 0;

    Q_INVOKABLE virtual Ovi::StoreItemList* newAlbumReleases() = 0;
    Q_INVOKABLE virtual Ovi::StoreItemList* newSingleRelease() = 0;
    Q_INVOKABLE virtual Ovi::StoreItemList* newTrackRelease() = 0;
    Q_INVOKABLE virtual Ovi::StoreItemList* topAlbums() = 0;
    Q_INVOKABLE virtual Ovi::StoreItemList* topTracks() = 0;

    /*!
     * Returns a list of playlists available.
     */
    Q_INVOKABLE virtual Ovi::Playlists* playlists() = 0;

    /*!
     * Returns a list of available subscriptions.
     */
    Q_INVOKABLE virtual Ovi::Subscriptions* subscriptions() = 0;

    /*!
     * Returns the account object.
     *
     * The returned Account object is owned by the store.
     *
     * If called before the store is ready, returns 0.
     */
    /*Q_INVOKABLE*/ virtual Ovi::Account* account() = 0;

    virtual RequestStatus *status() const = 0;
};

} /* end Ovi namespace */

#endif // STORE_H
