/**
  * Button prodives the button styles from the Meego toolkit.
  * A width is required, please see your design for specified width
  *
  * Properties for a button include:
  *     - text (string)
  *     - checked: true/false
  *     - disabled: true/false
  *
  */


import Qt 4.7

BorderImage {
    id: button

    property string productId: ""
    property bool isTrackDownload: false
    property bool isAlbumDownloadOnlyTrack: false
    property alias text: label.text
    property int padding: 28

    property bool downloadInProgress: false
    property bool playable: false
    property bool disabled: false
    property bool checked: false
    property string buttonState: (disabled) ?
                                 ((checked) ? "checkedDisabled" : "disabled") :
                                 ((checked) ? "checked" : "normal")

    signal clicked


    state:  {
        if (downloadEngine.albumDownloadData[productId]) {
            return downloadEngine.albumDownloadData[productId].status;
        }

        else if(downloadEngine.downloadData[productId]) {
            // When download button is tapped for this product.
            if(downloadEngine.downloadData[productId].status === "play" && mouseArea.pressed === true && !disabled) {
                // Download is finished and play button tapped, we show the "pressed" state
                return "playPressed";
            } else if(downloadEngine.downloadData[productId].status === "queuingDownload" && isAlbumDownloadOnlyTrack) {
                // If track is album download only state isn't changed to "queuingDownload"
                return "albumDownloadOnlyTrack";
            } else if(downloadEngine.downloadData[productId].status) {
                return downloadEngine.downloadData[productId].status;
            }
        }

        else if (isAlbumDownloadOnlyTrack) {
            return "albumDownloadOnlyTrack";
        }

        else {
            // When download button is not tapped for this product.
            if (mouseArea.pressed === true && !disabled) {
                 return 'pressed';
            } else {
                 return buttonState;
            }
        }
    }

    width: label.paintedWidth + padding
    height: 28
    smooth: true

    border { left: 17; top: 14; right: 17; bottom: 14 }
    horizontalTileMode: BorderImage.Stretch
    verticalTileMode: BorderImage.Stretch


        Text {
            id: label

            anchors.horizontalCenter: parent.horizontalCenter
            anchors.verticalCenter: parent.verticalCenter
            //font.capitalization: Font.AllUppercase
            font.pixelSize: 18
            font.family: theme.primaryFont
            font.bold: false
            smooth: true
        }

        NumberAnimation on rotation {
            id: animation
            from: 0;
            to: 360;
            loops: Animation.Infinite;
            duration: 1200;
            running: false
        }

    MouseArea {
        id: mouseArea
        anchors.fill: parent

        onClicked: {
            if (!button.disabled && !button.downloadInProgress) {
                button.clicked();
            }
        }
    }

    states: [
        State {
            // Default state for download button, clicking triggers download flow
            name: "normal"
            PropertyChanges { target: button; source: '../../images/buttons/btnFullNormalSmall.png';}
            PropertyChanges { target: label; color: '#282828';}
            PropertyChanges { target: animation; running: false }
            PropertyChanges { target: mouseArea; enabled: true}
        },
        State {
            name: "checked"
            PropertyChanges { target: button; source: '../../images/buttons/btnFullSelected.png';}
            PropertyChanges { target: label; color: '#FFFFFF';}
        },
        State {
            name: "pressed"
            PropertyChanges { target: button; source: '../../images/buttons/btnFullPressedSmall.png';}
            PropertyChanges { target: label; color: '#717177';}
            PropertyChanges { target: animation; running: false;}

        },
        State {
            // Fatal error state for download button, error message visibility bound to this state
            name: "disabled"
            PropertyChanges { target: animation; running: false }
            PropertyChanges { target: button; source: '../../images/buttons/btnFullDisabledSmall.png';}
            PropertyChanges { target: label; color: '#999A9C';}
            PropertyChanges { target: button; disabled: true }
            PropertyChanges { target: mouseArea; enabled: false}
        },
        State {
            name: "checkedDisabled"
            PropertyChanges { target: button; source: '../../images/buttons/btnFullSelectedDisabled.png';}
            PropertyChanges { target: label; color: '#F1F2F3';}
        },
        State {
            // "Recoverable error state for download button, error message visibility bound to this state
            name: "temporarilyDisabled"
            extend: "disabled"
            StateChangeScript {
                name: "stateChangeScript"
                script: stateChangeTimer.start();
            }
        },
        State {
            // Downloading state, download is being processed in DownloadManager
            name: "wait"
            //PropertyChanges { target: button; source: '../../images/wait.png';  }
            //PropertyChanges { target: button; height: 25; width: 25 }
            //PropertyChanges { target: label; opacity: 0}
            //PropertyChanges { target: animation; running: true }
            PropertyChanges { target: button; downloadInProgress: true }
            PropertyChanges { target: button; source: '../../images/buttons/btnFullSelected.png';}
            PropertyChanges { target: label; color: '#FFFFFF';}
        },
        State {
            // State after succesfull download, clicking triggers track play in native MusicPlayer
            // TODO: Finalize style for "Play" -button
            name: "play"
            PropertyChanges { target: button; source: '../../images/buttons/btnFullNormalSmall.png';}
            PropertyChanges { target: label; color: '#282828';}
            PropertyChanges { target: label; text: "Play" }
            PropertyChanges { target: button; playable: true }
            PropertyChanges { target: mouseArea; enabled: true}
        },
        State {
            // TODO: Finalize style for "Play" -button
            name: "playPressed"
            PropertyChanges { target: button; source: '../../images/buttons/btnFullPressedSmall.png';}
            PropertyChanges { target: label; color: '#717177';}
            PropertyChanges { target: label; text: "Play" }
            PropertyChanges { target: button; playable: true }
        },
        State {
            // Download is added to DownloadManager to be processed
            name: "queuingDownload"
            extend: "disabled"
        },
        State {
            // TODO: Finalize style for "AlbumDownloadOnly" -button
            // State for buttons of tracks that are only available as part of the album download
            name: "albumDownloadOnlyTrack"
            PropertyChanges { target: button; text: "( ! )"; }
            PropertyChanges { target: mouseArea; enabled: true; }
        },
        State {
            // TODO: Finalize style for "AlbumDownloadOnly" -button
            // State for buttons of tracks that are only available as part of the album download
            name: "albumDownloadOnlyTrackPressed"
            extend: "albumDownloadOnlyTrack"
            PropertyChanges { target: mouseArea; enabled: false; }
            StateChangeScript {
                name: "stateChangeScript"
                script: stateChangeTimer.start();
            }
        }
    ]


    Timer {
        id: stateChangeTimer
        interval: 3000
        repeat: false
        running: false
        onTriggered: {
            if (button.isTrackDownload && !button.isAlbumDownloadOnlyTrack) {
                downloadEngine.updateTrackStatusData(productId, "normal");
            } else if (button.isTrackDownload && button.isAlbumDownloadOnlyTrack) {
                downloadEngine.updateTrackStatusData(productId, "albumDownloadOnlyTrack");
            } else {
                downloadEngine.updateAlbumStatusData(productId, "normal", "normal");
            }
        }
    }
}
