/**
  * Button prodives the button styles from the Meego toolkit.
  * A width is required, please see your design for specified width
  *
  * Properties for a button include:
  *     - text (string)
  *     - align (default is center aligned by sometime you will need a button with left aligned text)
  *     - leftPadding and rightPadding (integer set to 26 by default)
  *
  *     - checked: true/false
  *     - disabled: true/false
  *
  *     - buttonPos: 'normal', 'left', 'middle' or 'right' used for button rows
  *
  *     - iconPos: 'left' 'center' or 'right' (default is set to center)
  *     - iconSource: specify a path to the image as a string
  *     - checkedIconSource: as above, if not speficed iconSource is used
  *     - disabledIconSource: as above
  *     - pressedIconSource: as above
  *
  */

import Qt 4.7

BorderImage {
    id: button

    property alias text: label.text
    property alias align: label.horizontalAlignment
    property alias font: label.font

    property int leftPadding: 26
    property int rightPadding: 26

    property bool disabled: false
    property bool checked: false
    property string buttonPos: 'normal'
    property string buttonState: (disabled) ?
                                     ((checked) ? "checkedDisabled" : "disabled") :
                                     ((checked) ? "checked" : "normal")

    signal clicked

    // Icon properties. Precedence is the following:
    // Source has precedence over Id
    // When checked, try to use checked Source or Id, if empty, fallback to default Source or Id
    property string iconPos: 'center'
    property string iconSource
    property string checkedIconSource
    property string disabledIconSource
    property string pressedIconSource

    state: (mouseArea.pressed === true && !disabled) ? 'pressed' : buttonState
    width: 190
    height: 52

    border {left: 25; top: 25; right: 25; bottom: 25 }

    horizontalTileMode: BorderImage.Stretch
    verticalTileMode: BorderImage.Stretch


        Text {
            id: label

            font.capitalization: Font.AllUppercase
            font.pixelSize: 22
            font.family: theme.primaryFont
            font.bold: true
            smooth: true
            elide: Text.ElideRight

            anchors {
                fill: parent
                leftMargin: leftPadding
                rightMargin: rightPadding
            }

            horizontalAlignment: Text.AlignHCenter
            verticalAlignment: Text.AlignVCenter
            visible: (iconPos === 'center' && iconSource === undefined) ? false : true
        }

        Image {
            id: icon
            source: iconSource

            anchors {
                left: (iconPos === 'left') ? parent.left : undefined
                right: (iconPos === 'right') ? parent.right : undefined
                margins: 17
                horizontalCenter: (iconPos === 'center') ? parent.horizontalCenter : undefined
                verticalCenter: parent.verticalCenter
            }
        }

        Image {
            //anchors.left: button.right -1
            //anchors.leftMargin: -1
            anchors.top: button.top
            source: '../../images/buttons/btnDivider.png'
            height: 52
            width: 1
            visible: (buttonPos === 'left' || buttonPos === 'normal') ? false : true
        }

    MouseArea {
        id: mouseArea
        anchors.fill: parent

        onPressed: {
            //button.state = 'pressed'
        }

        onClicked: {
            if (!button.disabled) {
                button.clicked();
            }
            //console.log(buttonState);
        }
    }

    states: [
            State {
                name: "normal"
                PropertyChanges {
                    target: button;
                    source: {
                        if(buttonPos === 'normal')
                            '../../images/buttons/btnFullNormal.png';
                        else if (buttonPos === 'left')
                            '../../images/buttons/btnLeftNormal.png';
                        else if (buttonPos === 'middle')
                            '../../images/buttons/btnMiddleNormal.png';
                        else if (buttonPos === 'right')
                            '../../images/buttons/btnRightNormal.png';
                    }
                }
                PropertyChanges { target: label; color: '#282828';}
            },
            State {
                name: "checked"
                PropertyChanges {
                    target: button;
                    //source: '../../images/buttons/btnFullSelected.png';
                    source: {
                        if(buttonPos === 'normal')
                            '../../images/buttons/btnFullSelected.png';
                        else if (buttonPos === 'left')
                            '../../images/buttons/btnLeftSelected.png';
                        else if (buttonPos === 'middle')
                            '../../images/buttons/btnMiddleSelected.png';
                        else if (buttonPos === 'right')
                            '../../images/buttons/btnRightSelected.png';
                    }

                }
                PropertyChanges { target: icon; source: (checkedIconSource) ? checkedIconSource : iconSource;}
                PropertyChanges { target: label; color: '#FFFFFF';}
            },
            State {
                name: "pressed"
                PropertyChanges {
                    target: button;
                    //source: '../../images/buttons/btnFullPressed.png';
                    source: {
                        if(buttonPos === 'normal')
                            '../../images/buttons/btnFullPressed.png';
                        else if (buttonPos === 'left')
                            '../../images/buttons/btnLeftPressed.png';
                        else if (buttonPos === 'middle')
                            '../../images/buttons/btnMiddlePressed.png';
                        else if (buttonPos === 'right')
                            '../../images/buttons/btnRightPressed.png';
                    }
                }
                PropertyChanges { target: icon; source: (pressedIconSource) ? pressedIconSource : iconSource;}
                PropertyChanges { target: label; color: '#717177';}
            },
             State {
                 name: "disabled"
                 PropertyChanges {
                     target: button;
                     //source: '../../images/buttons/btnFullNormalDisabled.png';
                     source: {
                         if(buttonPos === 'normal')
                             '../../images/buttons/btnFullNormalDisabled.png';
                         else if (buttonPos === 'left')
                             '../../images/buttons/btnLeftNormalDisabled.png';
                         else if (buttonPos === 'middle')
                             '../../images/buttons/btnMiddleNormalDisabled.png';
                         else if (buttonPos === 'right')
                             '../../images/buttons/btnRightNormalDisabled.png';
                     }
                 }
                 PropertyChanges { target: icon; source: (disabledIconSource) ? disabledIconSource : iconSource;}
                 PropertyChanges { target: label; color: '#999A9C';}
             },
            State {
                name: "checkedDisabled"
                PropertyChanges {
                    target: button;
                    //source: '../../images/buttons/btnFullSelectedDisabled.png';
                    source: {
                        if(buttonPos === 'normal')
                            '../../images/buttons/btnFullSelectedDisabled.png';
                        else if (buttonPos === 'left')
                            '../../images/buttons/btnLeftSelectedDisabled.png';
                        else if (buttonPos === 'middle')
                            '../../images/buttons/btnMiddleSelectedDisabled.png';
                        else if (buttonPos === 'right')
                            '../../images/buttons/btnRightSelectedDisabled.png';
                    }
                }
                PropertyChanges { target: icon; source: (disabledIconSource) ? disabledIconSource : iconSource;}
                PropertyChanges { target: label; color: '#F1F2F3';}
            }
         ]


}
