import Qt 4.7

import "../../javascript/trackDataHelper.js" as JS

/**
 * Contains the track's prefix, name and progress bar. Binds directly to music player's
 * properties to decide whether the progress bar should be shown and what is the state
 * of it. In case of music player error, shows the error note and hides it with a timer.
 */
Item {
    id: trackItem

    property alias trackPrefix: labelPrefix.text
    property alias trackName: label.text
    property alias artistName: artistLabel.text;
    property bool variousArtists;
    property int trackLengthInSec
    property string url

    // Decide whether to show the player based on music player's state and current track
    property bool playerVisible: musicPlayer.isPlaying && musicPlayer.currentTrack == url
    //property bool playerVisible: true;
    property int playerHeight: (variousArtists) ? 100 : 100
    property int nonPlayerHeight: (variousArtists) ? 76 : 64

    // Listen to errors from music player. If this track was playing, show the error state.
    property string error: musicPlayer.error
    property int progressInMillis: musicPlayer.progress
    property int cEND_OF_MEDIA: musicPlayer.cEND_OF_MEDIA

    onErrorChanged: {
        if (trackItem.state == "playerVisible") {
            trackItem.state = "errorShown";
            // TODO: track doesn't need to be stopped, it happens automatically in the case of errors, just used for testing
            musicPlayer.stop();
        }
    }
    onProgressInMillisChanged: JS.setProgressBarValue(progressInMillis, trackLengthInSec)


    Rectangle {
        id: themeBorder
        visible: playerVisible? true: false
        height: container.height
        width: 6
        color: theme.themeColor

    }

    BodyText {
        id: labelPrefix
        anchors {
            left: parent.left;
            leftMargin: 16;
            top: parent.top
            topMargin: 18
        }
        width: paintedWidth
        font.bold: true
        color: theme.tertiaryColor
    }

    BodyText {
        id: label
        anchors {
            left: labelPrefix.right
            top: parent.top
            topMargin: 18
        }
        width: parent.width - labelPrefix.width - priceButton.width - 50 //50 is the collective padding
        font.bold: true
        elide: Text.ElideRight
    }

    BodyText {
        id: artistLabel
        visible: (variousArtists && !playerVisible) ? true : false
        height: (visible) ? 25 : 0
        anchors {
            top: label.bottom;
            left: label.left;
            topMargin: 2
        }
        width: parent.width - priceButton.width - 50
        font.pixelSize: 22
        color: theme.tertiaryColor
        elide: Text.ElideRight
    }

    ClipPlayer {
        id: player
        buffering: playerVisible && musicPlayer.buffering
        width: parent.width
        anchors {
            top: artistLabel.bottom;
            topMargin: 6;
        }
        opacity: 0
    }

    BodyText {
        id: errorText
        text: qsTranslate("MusicPlayer","t[ErrorNote")
        width: parent.width
        font.italic: true
        color: theme.tertiaryColor
        elide: Text.ElideRight
        anchors {
            top: artistLabel.bottom;
            topMargin: 10;
            left: parent.left;
            leftMargin: 16;
            right: parent.right;
            rightMargin: 16;
        }
        opacity: 0
    }

    Timer {
        id: hideErrorTimer
        interval: 3000
        onTriggered: trackItem.state = "playerHidden"
    }

    states: [
        State {
            name: "playerHidden"
            // The "url &&" prevents the first invisible item to get the original height (see ProcutView:253)
            when: url && !playerVisible && !hideErrorTimer.running
        },
        State {
            name: "playerVisible"
            when: playerVisible
        },
        State {
            name: "errorShown"
            PropertyChanges {
                target: hideErrorTimer
                running: true
            }
            PropertyChanges {
                target: label
                color: theme.tertiaryColor
            }
            PropertyChanges {
                target: priceButton // References directly the priceButton in TrackListItem.qml
                state: "disabled"
            }
        }
    ]

    transitions: [
        Transition {
            to: "playerVisible"
            NumberAnimation { target: container; property: "height"; to: playerHeight; duration: 100 }
            NumberAnimation { target: errorText; property: "opacity"; to: 0; duration: 50 }
            SequentialAnimation {
                PauseAnimation { duration: 30 }
                NumberAnimation { target: player; property: "opacity"; to: 1; duration: 50 }
            }
        },
        Transition {
            to: "playerHidden"
            NumberAnimation { target: player; property: "opacity"; to: 0; duration: 50 }
            NumberAnimation { target: errorText; property: "opacity"; to: 0; duration: 50 }
            NumberAnimation { target: container; property: "height"; to: nonPlayerHeight; duration: 100 }
        },
        Transition {
            to: "errorShown"
            NumberAnimation { target: container; property: "height"; to: playerHeight; duration: 50 }
            NumberAnimation { target: player; property: "opacity"; to: 0; duration: 100 }
            NumberAnimation { target: errorText; property: "opacity"; to: 1; duration: 100 }
        }

    ]
}
