import Qt 4.7
import "constants.js" as C
import "AssetLoader.js" as AL

/*
   Class: Button
   button component that has a label and has click event handling.

   A button is a component that accepts user input and send a click() event for
   the application to handle. The button has resizable properties, event
   handling, and can undergo state changes and transitions.

   Note that you can put inside this components icons with size up to 40x40 pixels, but the preffered size is 32x32 pixels.

   <code>
       // Create a button with different icon states:
       // This approach works for all supported states: normal, disabled, pressed, selected, selected && disabled
       Button {
           text: "Jenson Button"
           iconSource: "assets/icon_" + (pressed ? "pressed" : "normal") + ".png"
       }
   </code>
*/
ImplicitSizeItem {

    /*
     * Property: text
     * [string] The text displayed on button.
     */
    property string text: ""

    /*
     * Property: iconSource
     * [url] The path to the icon image to be used. If you want a different icon for the checked state, you can
     * write iconSource: (checked) ? "path to icon checked" : "path to normal icon".
     */
    property url iconSource: ""

    /*
     * Property: checked
     * [bool] Specify the checked state. Its only working if checkable is set to true
     */
    property bool checked: false

    /*
     * Property: checkable
     * [bool=false] Specify the stateful behavior. The checked property is only working if true.
     */
    property bool checkable: false

    /*
     * Property: enabled
     * [bool] Enables/Disables the component. Notice that the disable state is not Toolkit compliant
     * but is supported on qt-components as well.
     */
    property bool enabled: true

    /*
     * Property: pressed
     * [bool] (ReadOnly) Is true when the button is pressed
     */
    property bool pressed: (state === 'pressed')

    /*
     * Event: clicked
     * Is emitted after the button is released
     */
    signal clicked


    /* private */
    property string __family: "meegotouch-button"
    property string __bgTemplate: "meegotouch-button{type}-background{disabled}{state}"
    property int __iconSize: 40
    property string __type: ""
    property alias __icon: icon
    property alias __text: label

    id: button

    implicitHeight: C.SIZE_BUTTON
    implicitWidth: __calculateWidth()

    state: {
        if (mouse.pressed) {
            'pressed'
        } else if (!enabled) {
            (checkable && checked) ? 'selected' : 'disabled'
        } else if (checkable) {
            checked ? 'selected' : ''
        } else {
            ''
        }
    }

    BorderImage {
        border.left: 20; border.top: 20
        border.right: 20; border.bottom: 20
        anchors.fill: parent
        source: __getBackground()
    }

    MouseArea {
        id: mouse
        anchors.fill: parent
        enabled: parent.enabled
        onClicked: {
            checked = !checked
            parent.clicked()
        }
    }

    Image {
        id: icon
        anchors.left: (label.visible && __type != "tabbarbutton") ? parent.left : undefined
        anchors.verticalCenter: parent.verticalCenter
        anchors.horizontalCenter: (label.visible) ? undefined : parent.horizontalCenter
        anchors.leftMargin: __getIconMargin()

        height: sourceSize.height * __getImageRatio()
        width: sourceSize.width * __getImageRatio()

        source: __getIcon()
        visible: source != ""

        property real __ratio

        function __getImageRatio() {
            if (!progress) return 1;
            return __ratio || (__ratio = Math.min(1, button.__iconSize / icon.sourceSize.height));
        }
    }

    Text {
        id: label
        anchors.verticalCenter: parent.verticalCenter
        anchors.left: (icon.visible) ? icon.right : parent.left
        anchors.right: parent.right
        anchors.leftMargin:  C.INDENT_DEFAULT
        anchors.rightMargin: C.INDENT_DEFAULT
        text: button.text
        color: __getTextColor()

        font.family: C.FONT_FAMILY
        font.pixelSize: C.FONT_DEFAULT_SIZE
        font.bold: C.FONT_BOLD_BUTTON
        font.capitalization: Font.AllUppercase
        horizontalAlignment: { (icon.visible) ? Text.AlignLeft : Text.AlignHCenter }
        visible: text != ""
        elide: Text.ElideRight

    }

    Text {
        id: labelSizeHelper
        font: label.font
        text: label.text
        visible: false
    }

    function __getIconMargin() {
        return C.INDENT_DEFAULT;
    }

    function __getTextColor() {
        if (__type === "") {
            if (state === "pressed") {
                return C.COLOR_BUTTON_SECONDARY_FOREGROUND;
            } else if (state === "disabled") {
                return (checkable && checked) ? C.COLOR_BUTTON_INVERTED_DISABLED_FOREGROUND: C.COLOR_BUTTON_DISABLED_FOREGROUND;
            } else if (state === "selected") {
                return C.COLOR_BUTTON_INVERTED_FOREGROUND;
            } else {
                return C.COLOR_BUTTON_FOREGROUND;
            }
        }
        // in any other case
        if (state === "pressed") {
            return C.COLOR_BUTTON_SECONDARY_FOREGROUND;
        } else if (state === "disabled") {
            return C.COLOR_BUTTON_INVERTED_DISABLED_FOREGROUND;
        } else {
           return C.COLOR_BUTTON_INVERTED_FOREGROUND;
        }

    }

    function __getBackground() {
        return AL.getId(__bgTemplate, {
                disabled: (!enabled && state !== "disabled"),
                state: state,
                type: __type,
                family: __family
        });
    }

    function __getIcon() {
        return iconSource
    }

    function __calculateWidth() {
        // XXX Check how does MeeGo Touch does that. Maybe use style paddings, etc
        var prefWidth = C.INDENT_DEFAULT * 2;
        // Fix the ASGARD-38 bug
        var iconWidth = icon.sourceSize.width * icon.__getImageRatio()
        if (label.visible && icon.visible) {
            prefWidth += labelSizeHelper.width + iconWidth + C.INDENT_DEFAULT;
        } else if (label.visible) {
            prefWidth += labelSizeHelper.width;
        } else if (icon.visible) {
            prefWidth += iconWidth;
        }

        return prefWidth;
    }
}
