import Qt 4.7

/*
   Class: DatePickerDialog
   Dialog that shows a date picker.

   WARNING:
   This component does not yet exist in qt-component, and the APIs are not yet agreed by the community.
   The final APIs may be different.
*/
Item {
    id: root

    /*
     * Property: title
     * [string] The title of the dialog.
     */
    property alias title: tumblerDialog.title

    /*
     * Property: minimumYear
     * [int] Optional, the minimum year shown on the tumbler.
     */
    property int minimumYear: 0

    /*
     * Property: maximumYear
     * [int] Optional, the maximum year shown on the tumbler.
     */
    property int maximumYear: 0

    /*
     * Property: acceptButtonLabel
     * [string] Optional, the button label for the accept button.
     */
    property alias acceptButtonLabel: tumblerDialog.acceptButtonLabel

    /*
     * Property: rejectButtonLabel
     * [string] Optional, the button label for the reject button.
     */
    property alias rejectButtonLabel: tumblerDialog.rejectButtonLabel

    /*
     * Property: deleteOnClose
     * [bool=false] Set to true if the dialog is created by a Loader, and should self destroy.
     *              when it is hided. Leave it at false otherwise.
     */
    property alias deleteOnClose: tumblerDialog.deleteOnClose

    /*
     * Event: selected
     * Is emitted when the user selects one of the buttons.
     */
    signal selected(string optionText, variant param)

    /*
     * Function: show
     * Makes the dialog visible.
     */
    function show(title, minimumYear, maximumYear, acceptButtonLabel, rejectButtonLabel) {
        if (typeof(title) != "undefined" && title != null) {
            root.title = title;
        }
        if (typeof(minimumYear) != "undefined" && minimumYear != null) {
            root.minimumYear = minimumYear;
        }
        if (typeof(maximumYear) != "undefined" && maximumYear != null) {
            root.maximumYear = maximumYear;
        }
        if (typeof(acceptButtonLabel) != "undefined" && acceptButtonLabel != null) {
            root.acceptButtonLabel = acceptButtonLabel;
        }
        if (typeof(rejectButtonLabel) != "undefined" && rejectButtonLabel != null) {
            root.rejectButtonLabel = rejectButtonLabel;
        }
        if (!internal.initialised) {
            _initializeDataModels();
        }
        tumblerDialog.component = tumblerComponent;
        tumblerDialog.show();
        if (internal.selected.length != 0) {
            setDate(internal.selected[0], internal.selected[1], internal.selected[2]);
            internal.selected = [];
        }
        if (internal.enabled.length != 0) {
            setEnabled(internal.enabled[0], internal.enabled[1]);
            internal.enabled = [];
        }
    }

    /*
     * Function: hide
     * hides the dialog.
     */
    function hide() {
        tumblerDialog.hide();
    }

    /*
     * Function: getDate
     * Gets the date selected by the user as a Javascript Date object. Value returned
     * is in UTC time, so use toUTCString() if you're displaying it.
     */
    function getDate() {
        if (internal.initialised) {
            return new Date(Date.UTC(
                yearsList.get(tumblerDialog.getSelected(2)).value,
                tumblerDialog.getSelected(1),
                tumblerDialog.getSelected(0) + 1)
            )
        }
        else {
            return null;
        }
    }

    /*
     * Function: setDate
     * Sets the date indicated by the "year", "month" and "date".
     * Alternatively, if the first argument is a Javascript Date object.
     * It will set the date to the same as the date object passed in.
     */
    function setDate(year, month, date) {
        if (internal.initialised) {
            var yearListCount = yearsList.count;
            if (typeof year == "object") {
                for (var i = 0; i < yearListCount; i++) {
                    if (yearsList.get(i).value == year.getFullYear()) {
                        tumblerDialog.setSelected([year.getDate() - 1, year.getMonth(), i]);
                    }
                }
            }
            else {
                for (var i = 0; i < yearListCount; i++) {
                    if (yearsList.get(i).value == year) {
                        tumblerDialog.setSelected([date - 1, month - 1, i]);
                    }
                }
            }
        }
        else {
            internal.selected = [year, month, date];
        }
    }

    /*
     * Function: getEnabled
     * Returns the enabled status of the column "index"
     */
    function getEnabled(index) {
        if (internal.initialised) {
            return tumblerDialog.getEnabled(index);
        }
        else {
            return null;
        }
    }

    /*
     * Function: setEnabled
     * Sets the enabled status of the column "index" with "value"
     */
    function setEnabled(index, value) {
        if (internal.initialised) {
            tumblerDialog.setEnabled(index, value);
        }
        else {
            internal.enabled = [index, value];
        }
    }

    function _initializeDataModels() {
        var currentYear = new Date().getFullYear();
        var min = minimumYear ? minimumYear : currentYear - 1;
        var max = maximumYear ? maximumYear : currentYear + 20;
        for (var year = min; year <= max; year++) { yearsList.append({"value" : year}) }
        for (var day = 1; day <= 31; day++) { daysList.append({"value" : day}) }
        for (var month = 1; month <= 12; month++) { monthsList.append({"value" : month}) }
        internal.initialised = true;
    }

    function _clearDataModels() {
        yearsList.clear();
        monthsList.clear();
        daysList.clear();
    }

    anchors.fill: parent
    onMinimumYearChanged: {
        _clearDataModels();
        internal.initialised = false;
    }
    onMaximumYearChanged: {
        _clearDataModels();
        internal.initialised = false;
    }

    TumblerDialogBase {
        id: tumblerDialog

        onSelected: root.selected(optionText, param)
    }

    Component {
        id: tumblerComponent

        Tumbler {
            id: tumbler

            function _handleTumblerChanges(index) {
                if (index == 1 || index == 2) {
                    var daysNo = [
                        [31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31],
                        [31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31]
                    ],
                    year = tumbler.getSelected(2) + yearsList.get(0).value,
                    month = tumbler.getSelected(1),
                    leapYear = ((year % 4 === 0) && (year % 100 !== 0) || (year % 400 === 0)) ? 1 : 0,
                    numberOfDays = daysNo[leapYear][month];

                    if (tumbler.getSelected(0) >= numberOfDays) { tumbler.setSelected(0, numberOfDays - 1) }
                    while (daysList.count > numberOfDays) { daysList.remove(daysList.count - 1) }
                    while (daysList.count < numberOfDays) { daysList.append({"value" : daysList.count + 1}) }
                }
            }

            items: [ daysList, monthsList, yearsList ]
            labels: [ "DAY", "MONTH", "YEAR" ]
            onChanged: _handleTumblerChanges(index);
            _delayInit: true
        }
    }

    ListModel {
        id: yearsList
    }

    ListModel {
        id: monthsList
    }

    ListModel {
        id: daysList
    }

    Item {
        id: internal

        property variant selected: []
        property variant enabled: []
        property variant initialised: false
    }
}
