import Qt 4.7
import "constants.js" as C

/*
   Class: ObjectMenu
   Dialog that allows the user to make a single selection.

   If used within a Loader and the dialog should self destruct when it's closed, set deleteOnClose to true

   WARNING:
   This component does not yet exist in qt-component, and the APIs are not yet agreed by the community.
   The final APIs may be different.
*/
Item {
    id: root

    /*
     * Property: title
     * [string] Optional, the title of the menu.
     */
    property string title

    /*
     * Property: subTitle
     * [string] Optional, the subTitle of the menu.
     */
    property string subTitle

    /*
     * Property: icon
     * [string] Optional, the path to the icon of the menu.
     */
    property string icon

    /*
     * Property: model
     * [ListModel] The model used for the contents of the menu.
     */
    property alias model: listView.model

    /*
     * Property: deleteOnClose
     * [bool=false] Set to true if the dialog is created by a Loader, and should self destroy.
     *              when it is hided. Leave it at false otherwise.
     */
    property bool deleteOnClose: false

    /*
     * Event: select
     * Is emitted when a selection is made.
     */
    signal selected(string optionText, variant param)

    /*
     * Function: show
     * Makes the dialog visible.
     */
    function show(model, title, subTitle, icon) {
        if (model) {
            root.model = model;
        }
        if (typeof(title) != "undefined" && title != null) {
            root.title = title;
        }
        if (typeof(subTitle) != "undefined" && subTitle != null) {
            root.subTitle = subTitle;
        }
        if (typeof(icon) != "undefined" && icon != null) {
            root.icon = icon;
        }
        dialog.height = dialog.getHeight();
        listView.positionViewAtIndex(0, ListView.Beginning);
        diaBg.state = "visible";
    }

    /*
     * Function: hide
     * hides the dialog.
     */
    function hide() {
        diaBg.state = "hidden";
    }

    anchors.fill: parent

    Item {
        id: internal

        property int maxHeight: 0

        onMaxHeightChanged: {
            dialog.height = dialog.getHeight()
        }
    }

    DialogBackground {
        id: diaBg

        onTransitionCompleted: {
            if (diaBg.state == "hidden" && root.deleteOnClose) {
                root.destroy();
            }
        }

        content:
            Item {
                id: dialog

                function getHeight() {
                    if (!listView.model) {
                        return internal.maxHeight;
                    }

                    var componentHeight;
                    if ((root.icon.length + root.title.length) > 0) {
                        componentHeight = 107 + (listView.model.count * 64);
                    } else {
                        componentHeight = 18 + (listView.model.count * 64);
                    }

                    return Math.min(internal.maxHeight, componentHeight);
                }

                height: getHeight()
                anchors { left: parent.left; right: parent.right; verticalCenter: parent.verticalCenter }

                Item {
                    id: topDivider

                    height: 65
                    anchors { top: parent.top; left: parent.left; right: parent.right; topMargin: 4 }

                    BorderImage {
                        source: "assets/meegotouch-objectmenu/meegotouch-button-objectmenu-background-vertical-top.png"
                        anchors.fill: parent
                        border { top: 2; bottom: 1; left: 1; right: 1 }
                    }
                }

                Item {
                    id: bottomDivider

                    height: 65
                    anchors { bottom: parent.bottom; left: parent.left; right: parent.right; bottomMargin: 4 }

                    BorderImage {
                        source: "assets/meegotouch-objectmenu/meegotouch-button-objectmenu-background-vertical-bottom.png"
                        anchors.fill: parent
                        border { top: 2; bottom: 1; left: 1; right: 1 }
                    }
                }

                Image {
                    id: titleRectDivider

                    visible: titleRect.visible
                    height: 1
                    anchors { top: titleRect.bottom; left: parent.left; right: parent.right; topMargin: 4 }
                    source: "assets/meegotouch-separator/meegotouch-separator-inverted-background-horizontal.png"
                }

                Rectangle {
                    id: titleRect

                    visible: (root.icon.length + root.title.length) > 0
                    height: 80
                    color: "transparent"
                    anchors { top: parent.top; left: parent.left; right: parent.right; topMargin: 9 }

                    Image {
                        id: tImage

                        visible: root.icon.length > 0
                        width: 64
                        height: 64
                        anchors { top: parent.top; left: parent.left; topMargin: 8; leftMargin: 16 }
                        source: root.icon
                    }

                    Text {
                        id: tTitle

                        visible: root.title.length > 0
                        color: "#FFFFFF"
                        font { pixelSize: 26; bold: true; family: C.FONT_FAMILY }
                        anchors { top: tsubTitle.visible ? parent.top : undefined; left: tImage.visible ? tImage.right : parent.left;
                            right: parent.right; topMargin: tsubTitle.visible ? 13: 0; leftMargin: 16; rightMargin: 16;
                            verticalCenter: tsubTitle.visible ? undefined : parent.verticalCenter }
                        text: root.title
                        elide: Text.ElideRight
                    }

                    Text {
                        id: tsubTitle

                        visible: root.subTitle.length > 0
                        color: "#FFFFFF"
                        font { pixelSize: 22; family: "Nokia Standard Light" }
                        anchors { bottom: parent.bottom; left: tImage.visible ? tImage.right : parent.left; right: parent.right;
                            bottomMargin: 10; leftMargin: 16; rightMargin: 16 }
                        text: root.subTitle
                        elide: Text.ElideRight
                    }

                    MouseArea {
                        anchors.fill: parent
                    }
                }

                ListView {
                    id: listView

                    function clicked(optionText, param) {
                        root.selected(optionText, param);
                        root.hide();
                    }

                    anchors { top: titleRect.visible ? titleRectDivider.bottom : parent.top; bottom: parent.bottom;
                        left: parent.left; right: parent.right; topMargin: titleRect.visible ? 4 : 9; bottomMargin: 9 }
                    clip: true
                    delegate: listViewDelegate
                }

                ScrollDecorator {
                    flickable: listView
                }

                Component {
                    id: listViewDelegate

                    Row {
                        height: 64
                        anchors { left: parent.left; right: parent.right }

                        Item {
                            // cannot anchor directly under row, need to have a buffer item here
                            height: parent.height
                            width: parent.width
                            Image {
                                anchors.fill: parent
                                fillMode: Image.Tile
                                source: listMA.pressed ?
                                        "assets/meegotouch-objectmenu/meegotouch-button-objectmenu-background-pressed-vertical-center.png" :
                                        "assets/meegotouch-objectmenu/meegotouch-button-objectmenu-background-vertical-center.png"
                            }
                            Text {
                                color: "#FFFFFF"
                                font { pixelSize: 26; bold: true; family: C.FONT_FAMILY }
                                anchors { left: parent.left; right: parent.right; leftMargin: 16; rightMargin: 16; verticalCenter: parent.verticalCenter}
                                text: optionText
                                elide: Text.ElideRight
                            }
                            MouseArea {
                                id: listMA

                                anchors.fill: parent
                                onClicked: listView.clicked(optionText, param)
                            }
                        }
                    }
                }
            }
    }

    states: [
        State {
            name: "landscape"
            // TODO runtime object is not universal
            //when: runtime.orientation == Orientation.Landscape || runtime.orientation == Orientation.LandscapeInverted
            when: root.width > root.height
            PropertyChanges {
                target: dialog
                anchors { leftMargin: root.width * 0.18; rightMargin: root.width * 0.18 }
            }
            PropertyChanges {
                target: internal
                maxHeight: (root.icon.length + root.title.length) > 0 ? root.height * 0.815 : root.height * 0.9
            }
        },
        State {
            name: "portrait"
            //when: runtime.orientation == Orientation.Portrait || runtime.orientation == Orientation.PortraitInverted
            // TODO runtime object is not universal
            when: root.width <= root.height
            PropertyChanges {
                target: dialog
                anchors { leftMargin: root.width * 0.04; rightMargin: root.width * 0.04 }
            }
            PropertyChanges {
                target: internal
                maxHeight: (root.icon.length + root.title.length) > 0 ? root.height * 0.83 : root.height * 0.88
            }
        }
    ]
}
