import Qt 4.7
import "constants.js" as C

/*
   Class: QueryDialog
   Dialog that queries the user for confirmation.

   If used within a Loader and the dialog should self destruct when it's closed, set deleteOnClose to true

   WARNING:
   This component does not yet exist in qt-component, and the APIs are not yet agreed by the community.
   The final APIs may be different.
*/
Item {
    id: root

    /*
     * Property: title
     * [string] The title of the dialog.
     */
    property string title

    /*
     * Property: text
     * [string] The text of the dialog.
     */
    property string text

    /*
     * Property: icon
     * [string] Optional, the path to the icon of the menu.
     */
    property string icon

    /*
     * Property: acceptButtonLabel
     * [string] Optional, the button label for the accept button.
     */
    property string acceptButtonLabel: "SAVE"

    /*
     * Property: rejectButtonLabel
     * [string] Optional, the button label for the reject button. Setting this to null will remove this button
     *          from the dialog
     */
    property string rejectButtonLabel: "CANCEL"

    /*
     * Property: deleteOnClose
     * [bool=false] Set to true if the dialog is created by a Loader, and should self destroy.
     *              when it is hided. Leave it at false otherwise.
     */
    property bool deleteOnClose: false

    /*
     * Event: selected
     * Is emitted when the user selects one of the buttons.
     */
    signal selected(string optionText, variant param)

    /*
     * Function: show
     * Makes the dialog visible.
     */
    function show(title, text, icon, acceptButtonLabel, rejectButtonLabel) {
        if (typeof(title) != "undefined" && title != null) {
            root.title = title;
        }
        if (typeof(text) != "undefined" && text != null) {
            root.text = text;
        }
        if (typeof(icon) != "undefined" && icon != null) {
            root.icon = icon;
        }
        if (typeof(acceptButtonLabel) != "undefined" && acceptButtonLabel != null) {
            root.acceptButtonLabel = acceptButtonLabel;
        }
        if (typeof(rejectButtonLabel) != "undefined" && rejectButtonLabel != null) {
            root.rejectButtonLabel = rejectButtonLabel;
        }
        dialog.height = dialog.getHeight();
        dFlickable.contentY = 0;
        diaBg.state = "visible";
    }

    /*
     * Function: hide
     * hides the dialog.
     */
    function hide() {
        diaBg.state = "hidden";
    }

    anchors.fill: parent
    onTextChanged: {
        dialog.height = dialog.getHeight();
    }
    onIconChanged: {
        dialog.height = dialog.getHeight();
    }

    Item {
        id: internal

        property int maxHeight: 0

        onMaxHeightChanged: {
            dialog.height = dialog.getHeight()
        }
    }

    DialogBackground {
        id: diaBg

        onTransitionCompleted: {
            if (diaBg.state == "hidden" && root.deleteOnClose) {
                root.destroy();
            }
        }
        modal: false
        content:
            Item {
                id: dialog

                function getHeight() {
                    var componentHeight;
                    if (root.icon.length > 0) {
                        componentHeight = (root.state == "landscape" ? 228 : 248) + dText.paintedHeight
                    } else {
                        componentHeight = (root.state == "landscape" ? 139 : 148) + dText.paintedHeight
                    }
                    return Math.min(internal.maxHeight, componentHeight);
                }

                function buttonClicked(text, param) {
                    root.selected(text, param);
                    root.hide();
                }

                height: getHeight()
                anchors { left: parent.left; right: parent.right; verticalCenter: parent.verticalCenter }

                Image {
                    id: tImage

                    visible: root.icon.length > 0
                    width: 80
                    height: 80
                    anchors { top: parent.top; horizontalCenter: parent.horizontalCenter }
                    source: root.icon
                    onVisibleChanged: dialog.height = dialog.getHeight()
                }

                Text {
                    id: tTitle

                    width: parent.width
                    color: "#1080DD"
                    font { pixelSize: 32; bold: true; family: C.FONT_FAMILY}
                    horizontalAlignment: Text.AlignHCenter
                    anchors { top: tImage.visible ? tImage.bottom : parent.top; topMargin: tImage.visible ? 20 : 0 }
                    text: root.title
                    elide: Text.ElideRight
                }

                Flickable {
                    id: dFlickable

                    interactive: dText.paintedHeight > height
                    contentWidth: width
                    contentHeight: dText.paintedHeight
                    anchors { top: tTitle.bottom; bottom: acceptButton.top; left: parent.left; right: parent.right;
                        topMargin: 21; bottomMargin: ((root.state == "landscape") && (root.icon.length == 0)) ? 48 : 37 }
                    flickableDirection: Flickable.VerticalFlick
                    boundsBehavior: Flickable.DragOverBounds
                    clip: true

                    Text {
                        id: dText

                        text: root.text
                        color: "#FFFFFF"
                        font { pixelSize: 24; family: "Nokia Standard Light" }
                        horizontalAlignment: Text.AlignHCenter
                        wrapMode: Text.WordWrap
                        anchors { top: parent.top; bottom: parent.bottom; left: parent.left; right: parent.right;
                            leftMargin: 24; rightMargin: 24 }
                        onHeightChanged: dialog.height = dialog.getHeight()
                    }
                }

                ScrollDecorator {
                    flickable: dFlickable
                }

                Button {
                    id: acceptButton

                    visible: root.rejectButtonLabel.length > 0
                    __type: "inverted"
                    text: root.acceptButtonLabel
                    anchors { bottom: parent.bottom; left: parent.left; right: buttonDivider.left;
                        leftMargin: dialog.width > 412 ? ((dialog.width - 6)/ 2) - 206 : root.width * 0.02 }
                    onClicked: dialog.buttonClicked(text, C.DIALOG_ACCEPT_BUTTON)
                }

                Item {
                    id: buttonDivider

                    width: 6
                    anchors { bottom: parent.bottom; horizontalCenter: parent.horizontalCenter }
                }

                Button {
                    id: rejectButton

                    visible: root.rejectButtonLabel.length > 0
                    __type: "inverted"
                    text: root.rejectButtonLabel
                    anchors { bottom: parent.bottom; left: buttonDivider.right; right: parent.right;
                        rightMargin: dialog.width > 412 ? ((dialog.width - 6)/ 2) - 206 : root.width * 0.02 }
                    onClicked: dialog.buttonClicked(text, C.DIALOG_REJECT_BUTTON)
                }

                Button {
                    // TODO shouldn't need a new button, but for some reason
                    // qmlviewer complains about
                    // QML Button: Cannot specify left, right, and hcenter anchors
                    // even when hcenter is set to undefined
                    id: middleButton

                    width: acceptButton.width
                    visible: root.rejectButtonLabel.length == 0
                    __type: "inverted"
                    text: root.acceptButtonLabel
                    anchors { bottom: parent.bottom; horizontalCenter: parent.horizontalCenter }
                    onClicked: dialog.buttonClicked(text, C.DIALOG_ACCEPT_BUTTON)
                }
            }
    }

    states: [
        State {
            name: "landscape"
            // TODO runtime object is not universal
            //when: runtime.orientation == Orientation.Landscape || runtime.orientation == Orientation.LandscapeInverted
            when: root.width > root.height
            PropertyChanges {
                target: dialog
                anchors { leftMargin: root.width * 0.24; rightMargin: root.width * 0.24 }
            }
            PropertyChanges {
                target: internal
                maxHeight: root.height * (root.icon.length > 0 ? 0.9 : 0.8)
            }
        },
        State {
            name: "portrait"
            //when: runtime.orientation == Orientation.Portrait || runtime.orientation == Orientation.PortraitInverted
            // TODO runtime object is not universal
            when: root.width <= root.height
            PropertyChanges {
                target: dialog
                anchors { leftMargin: root.width * 0.04; rightMargin: root.width * 0.04 }
            }
            PropertyChanges {
                target: internal
                maxHeight: root.height * 0.85
            }
        }
    ]
}
