import Qt 4.7

/*
   Class: RatingIndicator
   Component to indicate user specified ratings.

   A rating indicator is a component that shows the rating value within the maximum allowed range according
   to the user's specification.  The user can also specify the display type to select the positive/negative
   visual.  Optionally, the user can also specify a count value that will be displayed next to the images.

   Note: When "count" is used, the height of the entire indicator is different than when it is not used.
         This is due to the font size being larger than the image size.

   WARNING:
   This component does not yet exist in qt-component, and the APIs are not yet agreed by the community.
   The final APIs may be different.
*/
Item {
    id: ratingIndicator

    /*
     * Property: maximumValue
     * [double] The maximum rating.  The number should be larger or equal to 0.
     */
    property double maximumValue: 0

    /*
     * Property: ratingValue
     * [double] The rating value.  The number should be larger or equal to 0.
     */
    property double ratingValue: 0

    /*
     * Property: count
     * [int] A number to be displayed next to the rating images.  It is usually used to count the number of
     * votes cast. It is only displayed if a number larger or equal to 0 is specified.
     */
    property int count: -1

    /*
     * Property: inverted
     * [string] Specify whether the visual for the rating indicator uses the inverted color.  The value is
     * false for use with a light background and true for use with a dark background.
     */
    property bool inverted: false

    /* private */
    property int __imageWidth: 16
    property int __imageHeight: 16
    property int __horizontalSpace: 5  // space between images
    property int __indentSpace: 8  // space between image and text
    property string __backgroundImageSource: inverted ?
                                             "assets/meegotouch-rating-indicator/meegotouch-inverted-ratingindicator-ratingbackground.png" :
                                             "assets/meegotouch-rating-indicator/meegotouch-ratingindicator-ratingbackground.png"
    property string __ratingImageSource: inverted ?
                                         "assets/meegotouch-rating-indicator/meegotouch-inverted-ratingindicator-rating.png" :
                                         "assets/meegotouch-rating-indicator/meegotouch-ratingindicator-rating.png"

    property bool __init: true

    height: __calcHeight()
    width: __calcWidth()

    Component.onCompleted: {
        __updateUI();
        __init = false;
    }

    onRatingValueChanged: {
        if(!__init) {
            __updateUI();
        }
    }

    onCountChanged: {
        if(!__init) {
            __updateUI();
        }
    }

    function __updateUI() {

        if(maximumValue <= 0) {
            maximumValue = 0;
            ratingValue = 0;
            count = -1;
        }
        else {
            if(ratingValue < 0) {
                ratingValue = 0;
            }

            if(ratingValue > maximumValue) {
                ratingValue = maximumValue;
            }

            if(count < 0) {
                count = -1;
            }
        }

        // update visual elements
        if(count < 0) {
            text.visible = false;
            text.font.pixelSize = 1;  // make text smaller than image
        }
        else {
            text.visible = true;
            text.font.pixelSize = 18;
        }

        ratingBackground.width = __calcBackgroundWidth();

        if(ratingBackground.height > text.paintedHeight) {
            ratingBackground.anchors.verticalCenter = sizeHelper.verticalCenter
        }
        else {
            ratingBackground.anchors.verticalCenter = text.verticalCenter
        }

        rating.width = __calcRatingWidth();
    }

    function __calcHeight() {
        return Math.max(ratingBackground.height, text.paintedHeight);
    }

    function __calcWidth() {
        var rest = (count < 0) ? 0 : __indentSpace + text.paintedWidth;
        return ratingBackground.width + rest;
    }

    function __calcBackgroundWidth() {
        var maxSpaces = Math.max(Math.ceil(maximumValue-1), 0);
        return __imageWidth * maximumValue + maxSpaces * __horizontalSpace;
    }

    function __calcRatingWidth() {
        var numSpaces = Math.ceil(ratingValue) - 1;
        return __imageWidth * ratingValue + Math.max(numSpaces, 0) * __horizontalSpace;
    }

    Image {
        id: ratingBackground
        height: __imageHeight
        fillMode: Image.Tile
        source: __backgroundImageSource

        Image {
            id: rating
            height: __imageHeight
            fillMode: Image.Tile
            source: __ratingImageSource
        }
    }

    Item {
        id: sizeHelper
        height: __imageHeight
        width: __imageWidth
        visible: false
    }

    Text {
        id: text
        text: "(" + count + ")"
        color: inverted ? "#fafafa" : "#505050"
        font.family: "Nokia Standard Light"
        anchors.left: ratingBackground.right
        anchors.leftMargin: __indentSpace
    }
}
