import Qt 4.7
import "constants.js" as C
import "SelectionDialog.js" as PrivateImp

/*
   Class: SelectionDialog
   Dialog that allows the user to make a single selection.

   If highlight is needed, SelectionDialog should be used instead.
   If used within a Loader and the dialog should self destruct when it's closed, set deleteOnClose to true

   WARNING:
   This component does not yet exist in qt-component, and the APIs are not yet agreed by the community.
   The final APIs may be different.
*/
Item {
    id: root

    /*
     * Property: title
     * [string] The title of the dialog.
     */
    property string title

    /*
     * Property: model
     * [ListModel] The model used for the contents of the dialog.
     */
    property ListModel model

    /*
     * Property: selectedIndex
     * [int] Optional, the current selected index
     */
    property alias selectedIndex: listView.currentIndex

    /*
     * Property: showHighlight
     * [bool=false] true if the dialog should highlight the current item
     */
    property bool showHighlight: false

    /*
     * Property: multipleSelection
     * [bool=false] true if the dialog should allow multiple selection
     */
    property bool multipleSelection: false

    /*
     * Property: selectedItems
     * [array] Sets the selected items in multipleSelection mode
     *         TODO this is write only and cannot be read, due to a qt bug
     *         http://bugreports.qt.nokia.com/browse/QTBUG-9341
     *         To read, use getSelectedItems() for now
     */
    property variant selectedItems: []

    /*
     * Property: acceptButtonLabel
     * [string] Optional, the button label for the accept button.
     */
    property string acceptButtonLabel: "SAVE"

    /*
     * Property: rejectButtonLabel
     * [string] Optional, the button label for the reject button. Setting this to null will remove this button
     *          from the dialog
     */
    property string rejectButtonLabel: "CANCEL"


    /*
     * Property: deleteOnClose
     * [bool=false] Set to true if the dialog is created by a Loader, and should self destroy.
     *              when it is hided. Leave it at false otherwise.
     */
    property bool deleteOnClose: false

    /*
     * Event: selected
     * Is emitted when a selection is made.
     */
    signal selected(string optionText, variant param)

    /*
     * Function: getSelectedItems
     * Returns the selected items when multiple selection is enabled
     */
    function getSelectedItems() {
        var ret = [];
        var count = PrivateImp._origSelectedIndex.length;
        for (var i = 0; i < count; i++) {
            if (PrivateImp._origSelectedIndex[i]) {
                ret.push(i);
            }
        }
        return root.multipleSelection ? ret : [];
    }

    /*
     * Function: show
     * Makes the dialog visible.
     */
    function show(model, title, selectedIndex) {
        if (model) {
            root.model = model;
        }
        if (typeof(title) != "undefined" && title != null) {
            root.title = title;
        }
        if (typeof(selectedIndex) != "undefined" && selectedIndex != null) {
            root.selectedIndex = selectedIndex;
        }
        if (root.multipleSelection) {
            PrivateImp.backupIndexes(internalModel)
        }
        dialog.height = dialog.getHeight();
        listView.positionViewAtIndex(root.showHighlight ? root.selectedIndex : 0, ListView.Beginning);
        diaBg.state = "visible";
    }

    /*
     * Function: hide
     * hides the dialog.
     */
    function hide() {
        selectedItems[0] = true;
        diaBg.state = "hidden";
    }

    anchors.fill: parent
    onModelChanged: {
        if (root.model && (root.model.count > 0) && root.multipleSelection) {
            internal.handleModelChanged();
        } else if (!root.multipleSelection){
            listView.model = root.model
        }
    }
    onMultipleSelectionChanged: {
        if (root.model && (root.model.count > 0) && root.multipleSelection) {
            internal.handleModelChanged();
        } else if (!root.multipleSelection){
            listView.model = root.model
        }
    }
    onSelectedItemsChanged: {
        if (root.model && (root.model.count > 0) && root.multipleSelection) {
            internal.setSelectedItems();
        }
    }

    Item {
        id: internal

        property int maxHeight: 0

        function handleModelChanged() {
            var count = root.model.count;
            internalModel.clear();
            for (var i = 0; i < count; i++) {
                internalModel.append({"optionText" : model.get(i).optionText, "param" : model.get(i).param, "selected" : false});
            }
            setSelectedItems();
            listView.model = internalModel
        }

        function setSelectedItems() {
            var count = root.selectedItems.length;
            for (var i = 0; i < count; i++) {
                internalModel.setProperty(root.selectedItems[i], "selected", true);
            }
        }

        onMaxHeightChanged: {
            dialog.height = dialog.getHeight()
        }
    }

    DialogBackground {
        id: diaBg

        onTransitionCompleted: {
            if (diaBg.state == "hidden" && root.deleteOnClose) {
                root.destroy();
            }
        }

        modal: false
        content:
            Item {
                id: dialog

                function getHeight() {
                    if (!listView.model) {
                        return internal.maxHeight;
                    }
                    return Math.min(internal.maxHeight, 57 + (listView.model.count * 64) + (root.multipleSelection ? 50 : 0));
                }

                function buttonClicked(text, param) {
                    if (param == C.DIALOG_REJECT_BUTTON) {
                        PrivateImp.restoreIndexes(internalModel);
                    } else {
                        PrivateImp.backupIndexes(internalModel);
                    }
                    root.selected(text, param)
                    root.hide()
                }

                height: getHeight()
                anchors { left: parent.left; right: parent.right; verticalCenter: parent.verticalCenter }
                opacity: 0

                NumberAnimation on opacity {
                    id: animDialog

                    to: 1
                    duration: 400
                }

                Item {
                    id: tRect

                    height: 48
                    anchors { top: parent.top; left: parent.left; right: parent.right; }

                    Text {
                        id: tTitle

                        width: parent.width
                        color: "#1080DD"
                        font { pixelSize: 32; family: C.FONT_FAMILY }
                        anchors { top: parent.top; left: parent.left; right: tIcon.left; leftMargin: 11; rightMargin: 10 }
                        text: root.title
                        elide: Text.ElideRight
                    }

                    Image {
                        id: tIcon

                        width: 40
                        height: 40
                        source: "assets/icons/icon-m-common-dialog-close.png"
                        anchors { right: parent.right; verticalCenter: tTitle.verticalCenter }

                        MouseArea {
                            anchors.fill: parent
                            onClicked: root.hide()
                        }
                    }

                    Image {
                        width: parent.width
                        height: parent.height
                        fillMode: Image.Tile
                        source: "assets/meegotouch-dialogs/meegotouch-dialog-header-background.png"
                    }
                }

                Image {
                    anchors { bottom: root.multipleSelection ? acceptButton.top : parent.bottom; bottomMargin: 4 }
                    width: parent.width
                    height: 48
                    fillMode: Image.Tile
                    source: "assets/meegotouch-dialogs/meegotouch-dialog-background.png"
                }

                ListView {
                    id: listView

                    function clicked(optionText, param, index) {
                        if (!root.multipleSelection) {
                            root.selected(optionText, param);
                            root.hide();
                        } else {
                            internalModel.setProperty(index, "selected", !internalModel.get(index).selected);
                        }
                    }

                    anchors { top: tRect.bottom; bottom: root.multipleSelection ? acceptButton.top : parent.bottom;
                        left: parent.left; right: parent.right; bottomMargin: 9 }
                    clip: true
                    delegate: listViewDelegate
                    highlight: root.showHighlight && !root.multipleSelection ? highlightBar : null
                    highlightFollowsCurrentItem: true
                    highlightMoveDuration: 1
                    /*
                    // TODO why does this crashes on 360x640?
                    highlightRangeMode: ListView.ApplyRange
                    */
                }

                ScrollDecorator {
                    flickable: listView
                }

                Component {
                    id: listViewDelegate

                    Row {
                        height: 64
                        anchors { left: parent.left; right: parent.right }

                        Item {
                            // cannot anchor directly under row, need to have a buffer item here
                            height: parent.height
                            width: parent.width

                            Image {
                                anchors.fill: parent
                                fillMode: Image.Tile
                                source: root.multipleSelection ?
                                            // multiple selection true
                                            (internalModel.get(index).selected ?
                                                (listMA.pressed ?
                                                    "assets/meegotouch-objectmenu/meegotouch-button-objectmenu-background-selected-pressed-vertical-center.png":
                                                    "assets/meegotouch-objectmenu/meegotouch-button-objectmenu-background-selected.png") :
                                                (listMA.pressed ?
                                                    "assets/meegotouch-objectmenu/meegotouch-button-objectmenu-background-pressed-center.png":
                                                    "assets/meegotouch-objectmenu/meegotouch-button-objectmenu-background-center.png")) :
                                            // multiple selection false
                                            listMA.pressed ?
                                                (root.showHighlight ?
                                                     "assets/meegotouch-objectmenu/meegotouch-button-objectmenu-background-selected.png" :
                                                     "assets/meegotouch-objectmenu/meegotouch-button-objectmenu-background-pressed-center.png") :
                                                "assets/meegotouch-objectmenu/meegotouch-button-objectmenu-background-center.png"
                            }

                            Text {
                                color: "#FFFFFF"
                                font { pixelSize: 26; bold: true; family: C.FONT_FAMILY }
                                anchors { left: parent.left; right: parent.right; leftMargin: 16; rightMargin: 16;
                                    verticalCenter: parent.verticalCenter }
                                text: optionText
                                elide: Text.ElideRight
                            }

                            MouseArea {
                                id: listMA

                                anchors.fill: parent
                                onClicked: listView.clicked(optionText, param, index)
                            }
                        }
                    }
                }

                ListModel {
                    id: internalModel
                }

                Component {
                    id: highlightBar

                    Rectangle {
                        id: highlightBarRect
                        width: listView.width
                        height: 64

                        Image {
                            anchors.fill: parent
                            fillMode: Image.Tile
                            source: "assets/meegotouch-objectmenu/meegotouch-button-objectmenu-background-selected.png"
                        }
                    }
                }

                Button {
                    id: acceptButton

                    visible: root.multipleSelection
                    __type: "inverted"
                    text: root.acceptButtonLabel
                    anchors { bottom: parent.bottom; left: parent.left; right: buttonDivider.left;
                        leftMargin: dialog.width > 412 ? ((dialog.width - 6)/ 2) - 206 : root.width * 0.02 }
                    onClicked: dialog.buttonClicked(text, C.DIALOG_ACCEPT_BUTTON)
                }

                Item {
                    id: buttonDivider

                    width: 6
                    anchors { bottom: parent.bottom; horizontalCenter: parent.horizontalCenter }
                }

                Button {
                    id: rejectButton

                    visible: root.multipleSelection
                    __type: "inverted"
                    text: root.rejectButtonLabel
                    anchors { bottom: parent.bottom; left: buttonDivider.right; right: parent.right;
                        rightMargin: dialog.width > 412 ? ((dialog.width - 6)/ 2) - 206 : root.width * 0.02 }
                    onClicked: dialog.buttonClicked(text, C.DIALOG_REJECT_BUTTON)
                }
            }
    }

    states: [
        State {
            name: "landscape"
            // TODO runtime object is not universal
            //when: runtime.orientation == Orientation.Landscape || runtime.orientation == Orientation.LandscapeInverted
            when: root.width > root.height
            PropertyChanges {
                target: dialog
                anchors { leftMargin: root.width * 0.18; rightMargin: root.width * 0.18 }
            }
            PropertyChanges {
                target: internal
                maxHeight: root.height * 0.8
            }

        },
        State {
            name: "portrait"
            //when: runtime.orientation == Orientation.Portrait || runtime.orientation == Orientation.PortraitInverted
            // TODO runtime object is not universal
            when: root.width <= root.height
            PropertyChanges {
                target: dialog
                anchors { leftMargin: root.width * 0.04; rightMargin: root.width * 0.04 }
            }
            PropertyChanges {
                target: internal
                maxHeight: root.height * 0.85
            }
        }
    ]
}
