/****************************************************************************
**
** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
** All rights reserved.
** Contact: Nokia Corporation (qt-info@nokia.com)
**
** This file is part of the Qt Components project on Qt Labs.
**
** No Commercial Usage
** This file contains pre-release code and may not be distributed.
** You may use this file in accordance with the terms and conditions contained
** in the Technology Preview License Agreement accompanying this package.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 2.1 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU Lesser General Public License version 2.1 requirements
** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
** If you have questions regarding the use of this file, please contact
** Nokia at qt-info@nokia.com.
**
****************************************************************************/

import Qt 4.7
import "constants.js" as C

/*
   Class: Slider
   Generic slider component can be used when there is a need to adjust a value
   within a certain range, and the range can be intuitively represented along a
   linear scale. This scale presents the minimum and maximum values and optionally
   possible value levels between them. Slider is useful in situations where an immediate
   response for the value change is needed (e.g. music volume, brightness control, etc.)

   It's extended from <b>Item</b>.
*/
Item {
    id: root

    property int orientation: 0

    /*
     * Property: value
     * [real=0] Reflects the current value.
     */
    property real value: 0

    /*
     * Property: minimumValue
     * [real=0] Defines the lowest value you can select.
     */
    property real minimumValue: 0

    /*
     * Property: maximumValue
     * [real=100] Defines the highest value you can select.
     */
    property real maximumValue: 100
    property real stepSize: 1

    property bool enabled: true
    property bool updateValueWhileDragging: true
    property bool mousePressed: false
    property bool showTooltip: false

    // TODO seeker mode, not yet implemented
    property int indicatorPosition: 0
    property bool indicatorVisible: false
    property string indicatorLabel: ""
    // TODO why does this component has margins built in?
    property int innerLeftMargin: 0
    property int innerRightMargin: 0
    property int innerTopMargin: 0
    property int innerBottomMargin: 0
    // TODO this is not applicatable in asgard
    property real progress: 0

    signal pressed
    signal released
    // valueChanged is generated automatically by the runtime, no need to explicitly declare it

    width: orientation ? parent.height : parent.width
    height: 64
    onValueChanged: sliderContainer.updateValue()
    onMinimumValueChanged: sliderContainer.updateValue()
    onMaximumValueChanged: sliderContainer.updateValue()
    onPressed: root.mousePressed = true
    onReleased: root.mousePressed = false
    rotation: orientation ? 270: 0

    Item {
        id: sliderContainer

        function updateValue() {
            if (root.value > root.maximumValue) {
                root.value = root.maximumValue;
            } else if (root.value < root.minimumValue) {
                root.value = root.minimumValue;
            }
            if (!root.mousePressed) {
                sliderFill.width = ((root.value - root.minimumValue) / (root.maximumValue - root.minimumValue)) * sliderBackground.width;
                handle.x = sliderFill.width - (handle.width / 2);
            }
        }

        anchors { top: parent.top; bottom: parent.bottom; left: parent.left; right: parent.right;
            leftMargin: 20; rightMargin: 20}

        Item {
            id: sliderBackground

            width: parent.width
            height: 10
            anchors { horizontalCenter: parent.horizontalCenter; verticalCenter: parent.verticalCenter }
            onWidthChanged: sliderContainer.updateValue()

            MouseArea {
                enabled: root.enabled
                anchors.centerIn: parent
                width: parent.width
                height: 48 // parent is too thin, make it a bit taller
                onClicked: {
                    var val;
                    if (mouse.x == handleMouseArea.drag.maximumX) {
                        val = root.maximumValue; // for the case when max is not a multiple of step
                    } else {
                        val = root.minimumValue + ((mouse.x / sliderBackground.width) * (root.maximumValue - root.minimumValue));
                        val = Math.round(val / root.stepSize) * root.stepSize;
                        val = Math.max(val, root.minimumValue); // bound checking
                        val = Math.min(val, root.maximumValue); // bound checking
                    }
                    root.value = val;
                }
            }
        }

        BorderImage {
            id: sliderBackgroundImage

            anchors.fill: sliderBackground
            border { left: 4; top: 4; right: 4; bottom: 4 }
            smooth: true
        }

        Item {
            id: sliderFill

            width: 10 // need a small width to avoid rendering glitch
            height: sliderBackground.height
            anchors { top: sliderBackground.top; left: sliderBackground.left }
        }

        BorderImage {
            id: sliderFillImage

            visible: root.value != root.minimumValue
            anchors.fill: sliderFill
            border { left: 4; top: 4; right: 4; bottom: 4 }
            smooth: true
        }

        Item {
            id: toolTip

            visible: root.showTooltip && root.mousePressed
            width: ttText.width > 40 ? ttText.width + 20 : 40 // a little buffer on the side
            height: 32
            x: handle.x + ((handle.width - width) / 2)
            anchors.bottom: handle.top

            BorderImage {
                visible: toolTip.visible
                source: "assets/meegotouch-progress-indicators/meegotouch-slider-handle-value-background.png"
                anchors.fill: toolTip
                border { top: 14; bottom: 14; left: 18; right: 18; }
            }

            Image {
                anchors { top: parent.bottom; horizontalCenter: parent.horizontalCenter }
                source: "assets/meegotouch-progress-indicators/meegotouch-slider-handle-label-arrow-down.png"
            }

            Text {
                id: ttText

                color: "#FFFFFF"
                font.family: "Nokia Standard Light"
                font.pixelSize: 22
                anchors { horizontalCenter: parent.horizontalCenter; verticalCenter: parent.verticalCenter }
                text: root.value
            }
        }

        Image {
            id: handle

            width: 64
            height: 64
            x: -32
            anchors.verticalCenter: sliderBackground.verticalCenter

            MouseArea {
                id: handleMouseArea

                function onMouseMove() {
                    sliderFill.width = handle.x + (handle.width / 2);
                    if (root.updateValueWhileDragging) {
                        root.value = checkValue();
                    }
                }

                function onMousePressed() {
                    parent.source = "assets/meegotouch-progress-indicators/meegotouch-slider-handle-background-pressed-horizontal.png";
                    root.pressed();
                }

                function onMouseReleased() {
                    parent.source = "assets/meegotouch-progress-indicators/meegotouch-slider-handle-background-horizontal.png";
                    root.value = checkValue();
                    root.released();
                    sliderContainer.updateValue(); // force it to snap to grid
                }

                function checkValue() {
                    var val;
                    if (handle.x == handleMouseArea.drag.maximumX) {
                        val = root.maximumValue; // for the case when max is not a multiple of step
                    } else {
                        val = root.minimumValue +
                                (((handle.x + (handle.width / 2))/ sliderBackground.width) * (root.maximumValue - root.minimumValue));
                        val = Math.round(val / root.stepSize) * root.stepSize;
                        val = Math.max(val, root.minimumValue); // bound checking
                        val = Math.min(val, root.maximumValue); // bound checking
                    }
                    return val;
                }

                enabled: root.enabled
                anchors.fill: parent
                drag { target: parent; axis: Drag.XAxis; minimumX: -(parent.width / 2);
                    maximumX: sliderBackground.width - (parent.width / 2) }
                onPressed: onMousePressed()
                onReleased: onMouseReleased()
                onMousePositionChanged: onMouseMove()
            }
        }
    }

    states: [
        State {
            name: "enabled"
            when: enabled == true
            PropertyChanges {
                target: sliderBackgroundImage
                source: "assets/meegotouch-progress-indicators/meegotouch-slider-background-horizontal.png"
            }
            PropertyChanges {
                target: sliderFillImage
                source: "assets/meegotouch-progress-indicators/meegotouch-slider-elapsed-background-horizontal.png"
            }
            PropertyChanges {
                target: handle
                source: "assets/meegotouch-progress-indicators/meegotouch-slider-handle-background-horizontal.png"
            }
        },
        State {
            name: "disabled"
            when: enabled == false
            PropertyChanges {
                target: sliderBackgroundImage
                source: "assets/meegotouch-progress-indicators/meegotouch-slider-background-horizontal-disabled.png"
            }
            PropertyChanges {
                target: sliderFillImage
                source: "assets/meegotouch-progress-indicators/meegotouch-slider-elapsed-background-horizontal-disabled.png"
            }
            PropertyChanges {
                target: handle
                source: "assets/meegotouch-progress-indicators/meegotouch-slider-handle-background-horizontal-disabled.png"
            }
        }
    ]
}
