import Qt 4.7
import "constants.js" as C

/*
   Class: SmallButton
   Small button component that has a label and has click event handling. This button uses slightly differnt assets than a normal button.

   A button is a component that accepts user input and send a click() event for
   the application to handle. The button has resizable properties, event
   handling, and can undergo state changes and transitions.

   NOTE: This component is intended to be used inside List Items or View Header so by default provide the correct required size.
   SmallButton doesn't support an icon inside.

   WARNING: SmallButton exist today only in Ascard and could never be inside qt-components. Small Button and Normal Button are so similar that
   they could be merged in future toolkit releases. Use it at your own risk.

   <code>
       // Create a button with different icon states:
       // This approach works for all supported states: normal, disabled, pressed, selected, selected && disabled
       SmallButton {
           text: "Small"
       }
   </code>
*/
ImplicitSizeItem {
    /*
     * Property: text
     * [string] The text displayed on button.
     */
    property string text: ""

    /*
     * Property: enabled
     * [bool] Enables/Disables the component. Notice that the disable state is not Toolkit compliant
     * but is supported on qt-components as well.
     */
    property bool enabled: true

    /*
     * Property: pressed
     * [bool] (ReadOnly) Is true when the button is pressed
     */
    property bool pressed: (state === 'pressed')

    /*
     * Event: clicked
     * Is emitted after the button is released
     */
    signal clicked

    id: sbutton

    implicitHeight: C.SIZE_SMALL_BUTTON
    implicitWidth: C.WIDTH_SMALL_BUTTON

    states: [
        State {
            name: "pressed"
            when: mouse.pressed
            PropertyChanges {
                target: label
                color: C.COLOR_BUTTON_SECONDARY_FOREGROUND
            }
            PropertyChanges {
                target: borderimage
                source: "assets/meegotouch-textedit/meegotouch-textedit-button-background-pressed.png"
            }
        },
        State {
            name: "disabled"
            when: !sbutton.enabled
            PropertyChanges {
                target: label
                color: C.COLOR_BUTTON_DISABLED_FOREGROUND
            }
            PropertyChanges {
                target: borderimage
                source: "assets/meegotouch-textedit/meegotouch-textedit-button-background-disabled.png"
            }
        }
    ]

    BorderImage {
        border { left: 20; top: 20; right: 20; bottom: 20 }
        anchors.fill: parent
        source: "assets/meegotouch-textedit/meegotouch-textedit-button-background.png"
        id: borderimage
    }

    MouseArea {
        id: mouse
        anchors.fill: parent
        enabled: parent.enabled
        onClicked: parent.clicked()
    }

    Text {
        id: label

        anchors.verticalCenter: parent.verticalCenter
        anchors.left: parent.left
        anchors.right: parent.right
        anchors.leftMargin:  C.INDENT_DEFAULT
        anchors.rightMargin: C.INDENT_DEFAULT

        text: parent.text
        color: C.COLOR_BUTTON_FOREGROUND

        font.family: C.FONT_FAMILY
        font.pixelSize: C.FONT_DEFAULT_SIZE
        font.bold: C.FONT_BOLD_BUTTON
        font.capitalization: Font.AllUppercase
        horizontalAlignment: Text.AlignHCenter
        visible: text != ""
        elide: Text.ElideRight
    }

    Component.onCompleted: {
        console.log("WARNING: SmallButton exist today only in ascard and could never be inside qt-components.");
    }
}
