import Qt 4.7
import "constants.js" as C

/*
   Class: TumblerButton
   button component that has a label and has click event handling.

   A button is a component that accepts user input and send a click() event for
   the application to handle. The button has resizable properties, event
   handling, and can undergo state changes and transitions.

   <code>
       // Create a button with different icon states:
       // This approach works for all supported states: normal, disabled, pressed, selected, selected && disabled
       TumblerButton {
           text: "Tumbler Button"
       }
   </code>
*/
ImplicitSizeItem {

    /*
     * Property: text
     * [string] The text displayed on button.
     */
    property string text: ""

    /*
     * Property: enabled
     * [bool] Enables/Disables the component. Notice that the disable state is not Toolkit compliant
     * but is supported on qt-components as well.
     */
    property bool enabled: true

    /*
     * Property: pressed
     * [bool] (ReadOnly) Is true when the button is pressed
     */
    property bool pressed: (state === 'pressed')

    /*
     * Event: clicked
     * Is emitted after the button is released
     */
    signal clicked


    /* private */
    property string __bgTemplate: "meegotouch-button-background{disabled}{state}"
    property string __iconTemplate: "meegotouch-combobox-indicator{disabled}{state}"
    property int __iconSize: 32

    id: tumblerbutton

    implicitHeight: C.SIZE_BUTTON
    implicitWidth: __calculateWidth()

    state: {
        if (mouse.pressed) {
            'pressed'
        } else if (!enabled) {
            'disabled'
        } else {
            ''
        }
    }

    BorderImage {
        border.left: 20; border.top: 20
        border.right: 20; border.bottom: 20
        anchors.fill: parent
        source: __getAsset("assets/meegotouch-button/", __bgTemplate)
    }

    MouseArea {
        id: mouse
        anchors.fill: parent
        enabled: parent.enabled
        onClicked: {
            parent.clicked()
        }
    }

    Image {
        id: icon
        anchors.right: (label.text != "") ? parent.right : undefined
        anchors.verticalCenter: parent.verticalCenter
        anchors.horizontalCenter: (label.text != "") ? undefined : parent.horizontalCenter
        anchors.rightMargin: __getIconMargin()

        height: sourceSize.height
        width: sourceSize.width

        source: __getAsset("assets/meegotouch-button/", __iconTemplate)
        //source: "assets/meegotouch-button/meegotouch-combobox-indicator.png"
    }

    Text {
        id: label
        anchors.verticalCenter: parent.verticalCenter
        anchors.left: parent.left
        anchors.right: icon.left
        anchors.leftMargin:  C.INDENT_DEFAULT
        anchors.rightMargin: C.INDENT_DEFAULT
        text: tumblerbutton.text
        color: __getTextColor()

        font.family: C.FONT_FAMILY
        font.pixelSize: C.FONT_DEFAULT_SIZE
        font.bold: C.FONT_BOLD_BUTTON
        font.capitalization: Font.AllUppercase
        horizontalAlignment: Text.AlignLeft
        elide: Text.ElideRight
    }

    Text {
        id: labelSizeHelper
        font: label.font
        text: label.text
        visible: false
    }

    function __getIconMargin() {
        return C.INDENT_DEFAULT;
    }

    function __getTextColor() {
        if (state === "pressed") {
            return C.COLOR_BUTTON_SECONDARY_FOREGROUND;
        } else if (state === "disabled") {
            return C.COLOR_BUTTON_DISABLED_FOREGROUND;
        } else {
            return C.COLOR_BUTTON_FOREGROUND;
        }
    }

    function __getAsset(__path, __template) {
        var _disabled = (!enabled && state !== "disabled") ? "-disabled" : ""
        var _state = (state === "") ? "" : "-" + state
        return __path + __template.replace(/\{disabled\}/g, _disabled)
            .replace(/\{state\}/g, _state) + ".png";
    }

    function __calculateWidth() {
        // XXX Check how does MeeGo Touch does that. Maybe use style paddings, etc
        var prefWidth = C.INDENT_DEFAULT + 26; // see label left margin

        if (label.visible && icon.visible) {
            prefWidth += labelSizeHelper.width + icon.width + C.INDENT_DEFAULT;
        } else if (label.visible) {
            prefWidth += labelSizeHelper.width;
        } else if (icon.visible) {
            prefWidth += icon.width;
        }

        return prefWidth;
    }
}
