import Qt 4.7
import "constants.js" as C

/*
   Class: ViewMenu
   Dialog that allows the user to make a single selection.

   If used within a Loader and the dialog should self destruct when it's closed, set deleteOnClose to true

   WARNING:
   This component does not yet exist in qt-component, and the APIs are not yet agreed by the community.
   The final APIs may be different.
*/
Item {
    id: root

    /*
     * Property: model
     * [ListModel] The model used for the contents of the dialog.
     */
    property alias model: listView.model

    /*
     * Property: deleteOnClose
     * [bool=false] Set to true if the dialog is created by a Loader, and should self destroy.
     *              when it is hided. Leave it at false otherwise.
     */
    property bool deleteOnClose: false

    /*
     * Event: selected
     * Is emitted when a selection is made.
     */
    signal selected(string optionText, variant param)

    /*
     * Function: show
     * Makes the dialog visible.
     */
    function show(model) {
        if (model) {
            root.model = model;
        }
        dialog.height = dialog.getHeight();
        diaBg.state = "visible";
    }

    /*
     * Function: hide
     * hides the dialog.
     */
    function hide() {
        diaBg.state = "hidden";
    }

    anchors.fill: parent

    Item {
        id: internal

        property int maxHeight: 0

        onMaxHeightChanged: {
            dialog.height = dialog.getHeight()
        }
    }

    DialogBackground {
        id: diaBg

        onTransitionCompleted: {
            if (diaBg.state == "hidden" && root.deleteOnClose) {
                root.destroy();
            }
        }

        content:
            Item {
                id: dialog

                function getHeight() {
                    if (!listView.model) {
                        return internal.maxHeight;
                    }
                    return Math.min(internal.maxHeight, 4 + (listView.model.count * 64));
                }

                function buttonClicked(text) {
                    root.queryAnswered(text);
                    root.hide();
                }

                height: getHeight()
                anchors { bottom: parent.bottom; left: parent.left; right: parent.right }

                Image {
                    id: titleLine

                    width: parent.width
                    height: 1
                    anchors { top: parent.top; topMargin: 4 }
                    fillMode: Image.Tile
                    source: "assets/meegotouch-viewmenubuttons/meegotouch-viewmenu-background-vertical-top.png"
                }

                ListView {
                    id: listView

                    function clicked(optionText, param, index) {
                        root.selected(optionText, param)
                        root.hide()
                    }

                    anchors { top: titleLine.bottom; bottom: parent.bottom; left: parent.left; right: parent.right;
                        topMargin: 4; bottomMargin: 4 }
                    clip: true
                    delegate: listViewDelegate
                }

                ScrollDecorator {
                    flickable: listView
                }

                Component {
                    id: listViewDelegate

                    Row {
                        height: 64
                        anchors { left: parent.left; right: parent.right }

                        Item {
                            // cannot anchor directly under row, need to have a buffer item here
                            height: parent.height
                            width: parent.width

                            Image {
                                anchors.fill: parent
                                fillMode: Image.Tile
                                source: listMA.pressed ?
                                        "assets/meegotouch-objectmenu/meegotouch-button-objectmenu-background-pressed-center.png" :
                                        "assets/meegotouch-objectmenu/meegotouch-button-objectmenu-background-center.png"
                            }

                            Text {
                                color: "#FFFFFF"
                                font { pixelSize: 26; bold: true; family: C.FONT_FAMILY }
                                anchors { left: parent.left; right: parent.right; leftMargin: 16; rightMargin: 16
                                    verticalCenter: parent.verticalCenter }
                                text: optionText
                                elide: Text.ElideRight
                            }

                            MouseArea {
                                id: listMA

                                anchors.fill: parent
                                onClicked: listView.clicked(optionText, param, index)
                            }
                        }
                    }
                }
            }
    }

    states: [
        State {
            name: "landscape"
            // TODO runtime object is not universal
            //when: runtime.orientation == Orientation.Landscape || runtime.orientation == Orientation.LandscapeInverted
            when: root.width > root.height
            PropertyChanges {
                target: dialog
                anchors { leftMargin: root.width * 0.18; rightMargin: root.width * 0.18 }
            }
            PropertyChanges {
                target: internal
                maxHeight: root.height * 0.82
            }
        },
        State {
            name: "portrait"
            //when: runtime.orientation == Orientation.Portrait || runtime.orientation == Orientation.PortraitInverted
            // TODO runtime object is not universal
            when: root.width <= root.height
            PropertyChanges {
                target: dialog
                anchors { leftMargin: root.width * 0.04; rightMargin: root.width * 0.04 }
            }
            PropertyChanges {
                target: internal
                maxHeight: root.height * 0.84
            }
        }
    ]
}
